package weaver.interfaces.hostar.job;

import com.engine.hostar.util.HostarUtil;
import com.engine.kq.biz.KQGroupMemberComInfo;
import com.engine.kq.biz.KQHolidaySetBiz;
import com.engine.kq.biz.KQOvertimeRulesBiz;
import jdk.management.resource.ResourceId;
import org.apache.commons.lang3.StringUtils;
import org.apache.tools.ant.util.DateUtils;
import weaver.conn.RecordSet;
import weaver.general.BaseBean;
import weaver.general.Util;
import weaver.interfaces.schedule.BaseCronJob;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;

public class AddOvertimeSignJob extends BaseCronJob {

    private String fromDate;
    private String toDate;

    public String getFromDate() {
        return fromDate;
    }

    public String getToDate() {
        return toDate;
    }

    public void setFromDate(String fromDate) {
        this.fromDate = fromDate;
    }

    public void setToDate(String toDate) {
        this.toDate = toDate;
    }
    BaseBean bb= new BaseBean();

    @Override
    public void execute() {
        bb.writeLog("AddOvertimeSignJob Start");
        try {
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            DateTimeFormatter formatter2 = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

            bb.writeLog("fromDate: " + fromDate);
            bb.writeLog("toDate: " + toDate);

            if ( StringUtils.isBlank(fromDate) || StringUtils.isBlank(toDate)) {
                toDate = DateUtils.format(new Date(), "yyyy-MM-dd");

                LocalDate specifiedDate = LocalDate.parse(toDate, formatter);

                // 获取三天前的日期
                LocalDate threeDaysAgo = specifiedDate.minusDays(3);
                fromDate = threeDaysAgo.format(formatter);
            }
            bb.writeLog("--fromDate: " + fromDate);
            bb.writeLog("--toDate: " + toDate);

            RecordSet rs = new RecordSet();

            String overtimeWorkflow = Util.null2String(bb.getPropValue("project_hostar", "overtimeWorkflow"));
            bb.writeLog("overtimeWorkflow: " + overtimeWorkflow);

            if (StringUtils.isNotBlank(overtimeWorkflow)) {
                HostarUtil hostarUtil = new HostarUtil();
                List<String> allDates = hostarUtil.getAllDates(fromDate, toDate);
                bb.writeLog("allDates: " + allDates);

                for (String date: allDates) {
                    List<Map<String, Object>> overtimeInfo = new ArrayList<>();
                    //获取加班流程
                    String acqOvertimeSql = "select a.resourceId, a.fromDate, a.toDate, a.fromTime, a.toTime from " + overtimeWorkflow + " a " +
                            " left join workflow_requestbase b" +
                            " on b.requestid = a.requestid and b.currentnodetype = 3" +
                            " where a.fromDate = '" + date + "'";
                    bb.writeLog("acqOvertimeSql: " + acqOvertimeSql);
                    rs.executeQuery(acqOvertimeSql);
                    while (rs.next()) {
                        String resourceId = Util.null2String(rs.getString("resourceId"));
                        String fromDate = Util.null2String(rs.getString("fromDate"));
                        String fromTime = Util.null2String(rs.getString("fromTime"));
                        String toDate = Util.null2String(rs.getString("toDate"));
                        String toTime = Util.null2String(rs.getString("toTime"));
                        if (StringUtils.isNotBlank(fromDate) && StringUtils.isNotBlank(toDate) && StringUtils.isNotBlank(resourceId)) {
                            if (toDate.compareTo(fromDate) > 0 ) {
                                KQOvertimeRulesBiz kb = new KQOvertimeRulesBiz();
                                int changeType = kb.getChangeType(resourceId, fromDate);
                                bb.writeLog("changeType: " + changeType);
                                if (changeType != -1) {
                                    Map<String, Object> temp = new HashMap<>();
                                    temp.put("resourceId", resourceId);
                                    temp.put("fromDate", fromDate);
                                    temp.put("fromTime", fromTime);
                                    temp.put("toDate", toDate);
                                    temp.put("toTime", toTime);
                                    overtimeInfo.add(temp);
                                }
                            }
                        }
                    }

                    bb.writeLog("overtimeInfo: " + overtimeInfo);

                    //根据加班流程信息取打卡信息
                    for ( Map<String, Object> info: overtimeInfo) {
                        String resourceId = Util.null2String(info.get("resourceId"));
                        String fromDate = Util.null2String(info.get("fromDate"));
                        String fromTime = Util.null2String(info.get("fromTime"));
                        String toDate = Util.null2String(info.get("toDate"));
                        String toTime = Util.null2String(info.get("toTime"));

                        bb.writeLog("resourceId: " + resourceId + ", fromDate: " + fromDate + ", fromTime: " + fromTime + ", toDate: " + toDate + ", toTime: " + toTime);
                        // 将字符串时间转换为LocalDateTime对象
                        if (toTime.length() == 5) {
                            toTime = toTime + ":00";
                        }
                        LocalDateTime dateTime = LocalDateTime.parse(toDate + " " + toTime, formatter2);

                        // 获取指定时间前后三小时的时间范围
                        LocalDateTime startDateTimeL = dateTime.minus(3, ChronoUnit.HOURS);
                        LocalDateTime endDateTimeL = dateTime.plus(3, ChronoUnit.HOURS);

                        String startDateTime = startDateTimeL.format(formatter2);
                        bb.writeLog("startDateTime: " + startDateTime);

                        String endDateTime = endDateTimeL.format(formatter2);
                        bb.writeLog("endDateTime: " + endDateTime);

                        Integer number = -1;
                        String acqSignSql = "select count(*) as number from hrmschedulesign " +
                                " where signdate + ' '+signTime >='"+startDateTime+"' and signdate + ' '+signTime <='"+endDateTime+"' " +
                                " and userid = ? and signtype = 2 ";
                        bb.writeLog("--acqSignSql: " + acqSignSql);
                        rs.executeQuery(acqSignSql, resourceId);
                        while (rs.next()) {
                            number = Util.getIntValue(Util.null2String(rs.getString("number")));
                        }
                        bb.writeLog("number: " + number);

                        if (number > 0) {
                            //插入0点打卡
                            String addSign1Sql = "insert into hrmschedulesign (userId,userType,signType,signDate,signTime,isInCom,signfrom,timeZone,belongDate) " +
                                    " values (?,?,?,?,?,?,?,?,?)";
                            boolean b1 = rs.executeUpdate(addSign1Sql, resourceId, "1", "2", fromDate, "23:59:59", "1", "e9pc", "GMT+8", date);
                            bb.writeLog("插入23点59分打卡: " + b1);
                            //插入0点打卡
                            String addSign2Sql = "insert into hrmschedulesign (userId,userType,signType,signDate,signTime,isInCom,signfrom,timeZone,belongDate) " +
                                    " values (?,?,?,?,?,?,?,?,?)";
                            boolean b2 = rs.executeUpdate(addSign2Sql, resourceId, "1", "2", toDate, "00:00:00", "1", "e9pc", "GMT+8", date);
                            bb.writeLog("插入0点打卡: " + b2);
                        } else {
                            bb.writeLog("没有在指定时间范围内打下班卡，无法插入0点卡");
                        }

                    }

                }

            } else {
                bb.writeLog("没有设置加班流程");
            }
        }catch (Exception e) {
            bb.writeLog("AddOvertimeSignJob Exception: " + e);
        }
    }

}
